//This file is part of Furius Iso Mount. Copyright 2008 Dean Harris (marcus_furius@hotmail.com)
//
//    Furius Iso Mount is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    Furius Iso Mount is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Furius Iso Mount.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Security.Cryptography;
using System.IO;
using System.Text;
using System.Globalization;

namespace FuriusIsoMount
{
	
	
	/// <summary>
    /// Returns a hash value for a file
    /// </summary>
    public sealed class FileHash
    {

        /// <summary>
        /// Initializes a new instance of the <see cref="FileHash"/> class.
        /// </summary>
        private FileHash() { }

        public enum Provider { MD5, SHA1, SHA256, SHA384, SHA512 };

        /// <summary>
        /// Computes the hash.
        /// </summary>
        /// <param name="file">The file.</param>
        /// <param name="hashAlgorithm">The hash algorithm.</param>
        /// <returns></returns>
        public static string ComputeHash(string file, Provider hashAlgorithm)
        {
            Byte[] hashCode;
            using (FileStream fileStream = File.Open(file, FileMode.Open, FileAccess.Read))
            {
                switch (hashAlgorithm)
                {

                    case Provider.MD5:
                        using (MD5CryptoServiceProvider cryptoServiceProvider = new MD5CryptoServiceProvider())
                        {
                            hashCode = cryptoServiceProvider.ComputeHash(fileStream);
                        }
                        break;

                    case Provider.SHA1:
                        using (SHA1CryptoServiceProvider cryptoServiceProvider = new SHA1CryptoServiceProvider())
                        {
                            hashCode = cryptoServiceProvider.ComputeHash(fileStream);
                        }
                        break;
                    case Provider.SHA256:
                        using (SHA256Managed cryptoServiceProvider = new SHA256Managed())
                        {
                            hashCode = cryptoServiceProvider.ComputeHash(fileStream);
                        }
                        break;
                    case Provider.SHA384:
                        using (SHA384Managed cryptoServiceProvider = new SHA384Managed())
                        {
                            hashCode = cryptoServiceProvider.ComputeHash(fileStream);
                        }
                        break;
                    case Provider.SHA512:
                        using (SHA512Managed cryptoServiceProvider = new SHA512Managed())
                        {
                            hashCode = cryptoServiceProvider.ComputeHash(fileStream);
                        }
                        break;
                    default:
                        using (MD5CryptoServiceProvider cryptoServiceProvider = new MD5CryptoServiceProvider())
                        {
                            hashCode = cryptoServiceProvider.ComputeHash(fileStream);
                        }
                        break;
                }
            }
            StringBuilder stringBuilder = new StringBuilder();
            for (int i = 0; i < hashCode.Length; i++)
            {
                stringBuilder.Append(hashCode[i].ToString("x2", CultureInfo.InvariantCulture));
            }

            return stringBuilder.ToString();
        }
    }
}
